package java.util;

import java.util.function.Consumer;
import java.lang.ref.WeakReference;
import java.util.function.BiFunction;
import java.util.function.BiConsumer;
import java.lang.ref.Reference;
import java.lang.ref.ReferenceQueue;

public class WeakHashMap<K, V> extends AbstractMap<K, V> implements Map<K, V>
{
    private static final int DEFAULT_INITIAL_CAPACITY = 16;
    private static final int MAXIMUM_CAPACITY = 1073741824;
    private static final float DEFAULT_LOAD_FACTOR = 0.75f;
    WeakHashMap.Entry<K, V>[] table;
    private int size;
    private int threshold;
    private final float loadFactor;
    private final ReferenceQueue<Object> queue;
    int modCount;
    private static final Object NULL_KEY;
    private transient Set<Map.Entry<K, V>> entrySet;
    
    private WeakHashMap.Entry<K, V>[] newTable(int n) {
        return (WeakHashMap.Entry[])new WeakHashMap.Entry[n];
    }
    
    public WeakHashMap(int i, float f) {
        this.queue = new ReferenceQueue();
        if (i < 0) {
            throw new IllegalArgumentException("Illegal Initial Capacity: " + i);
        }
        if (i > 1073741824) {
            i = 1073741824;
        }
        if (f <= 0.0f || Float.isNaN(f)) {
            throw new IllegalArgumentException("Illegal Load factor: " + f);
        }
        int j;
        for (j = 1; j < i; j <<= 1) {}
        this.table = this.newTable(j);
        this.loadFactor = f;
        this.threshold = (int)((float)j * f);
        return;
    }
    
    public WeakHashMap(int n) {
        this(n, 0.75f);
    }
    
    public WeakHashMap() {
        this(16, 0.75f);
    }
    
    public WeakHashMap(Map<? extends K, ? extends V> map) {
        this(Math.max((int)((float)map.size() / 0.75f) + 1, 16), 0.75f);
        this.putAll(map);
    }
    
    private static Object maskNull(Object o) {
        return (o == null) ? WeakHashMap.NULL_KEY : o;
    }
    
    static Object unmaskNull(Object o) {
        return (o == WeakHashMap.NULL_KEY) ? null : o;
    }
    
    private static boolean eq(Object o, Object obj) {
        return o == obj || o.equals(obj);
    }
    
    final int hash(Object o) {
        int hashCode = o.hashCode();
        int n = hashCode ^ (hashCode >>> 20 ^ hashCode >>> 12);
        return n ^ n >>> 7 ^ n >>> 4;
    }
    
    private static int indexFor(int n, int n2) {
        return n & n2 - 1;
    }
    
    private void expungeStaleEntries() {
        Reference<Object> poll;
        while ((poll = this.queue.poll()) != null) {
            synchronized (this.queue) {
                WeakHashMap.Entry<Object, Object> entry = (WeakHashMap.Entry)poll;
                int index = indexFor(entry.hash, this.table.length);
                WeakHashMap.Entry<Object, Object> entry2 = this.table[index];
                while (entry2 != null) {
                    WeakHashMap.Entry<Object, Object> next = entry2.next;
                    if (entry2 == entry) {
                        if (entry2 == entry) {
                            this.table[index] = next;
                        }
                        else {
                            entry2.next = next;
                        }
                        entry.value = null;
                        --this.size;
                        break;
                    }
                    else {
                        entry2 = next;
                    }
                }
            }
        }
    }
    
    private WeakHashMap.Entry<K, V>[] getTable() {
        this.expungeStaleEntries();
        return this.table;
    }
    
    @Override
    public int size() {
        if (this.size == 0) {
            return 0;
        }
        this.expungeStaleEntries();
        return this.size;
    }
    
    @Override
    public boolean isEmpty() {
        return this.size() == 0;
    }
    
    @Override
    public V get(Object o) {
        Object maskNull = maskNull(o);
        int hash = this.hash(maskNull);
        WeakHashMap.Entry[] table = this.getTable();
        WeakHashMap.Entry<Object, Object> next = table[indexFor(hash, table.length)];
        while (next != null) {
            if (next.hash == hash && eq(maskNull, next.get())) {
                return next.value;
            }
            next = next.next;
        }
        return null;
    }
    
    @Override
    public boolean containsKey(Object o) {
        return this.getEntry(o) != null;
    }
    
    WeakHashMap.Entry<K, V> getEntry(Object o) {
        Object maskNull = maskNull(o);
        int hash = this.hash(maskNull);
        WeakHashMap.Entry[] table = this.getTable();
        WeakHashMap.Entry<Object, Object> next;
        for (next = table[indexFor(hash, table.length)]; next != null && (next.hash != hash || !eq(maskNull, next.get())); next = next.next) {}
        return next;
    }
    
    @Override
    public V put(K k, V value) {
        Object maskNull = maskNull(k);
        int hash = this.hash(maskNull);
        WeakHashMap.Entry[] table = this.getTable();
        int index = indexFor(hash, table.length);
        WeakHashMap.Entry<Object, Object> next = table[index];
        while (next != null) {
            if (hash == next.hash && eq(maskNull, next.get())) {
                Object value2 = next.value;
                if (value != value2) {
                    next.value = value;
                    return value2;
                }
                return value2;
            }
            else {
                next = next.next;
            }
        }
        ++this.modCount;
        table[index] = new WeakHashMap.Entry(maskNull, value, this.queue, hash, table[index]);
        if (++this.size >= this.threshold) {
            this.resize(table.length * 2);
        }
        return null;
    }
    
    void resize(int n) {
        WeakHashMap.Entry[] table = this.getTable();
        if (table.length == 1073741824) {
            this.threshold = Integer.MAX_VALUE;
            return;
        }
        WeakHashMap.Entry[] table2 = this.newTable(n);
        this.transfer(table, table2);
        this.table = table2;
        if (this.size >= this.threshold / 2) {
            this.threshold = (int)((float)n * this.loadFactor);
        }
        else {
            this.expungeStaleEntries();
            this.transfer(table2, table);
            this.table = table;
        }
        return;
    }
    
    private void transfer(WeakHashMap.Entry<K, V>[] array, WeakHashMap.Entry<K, V>[] array2) {
        for (int i = 0; i < array.length; ++i) {
            WeakHashMap.Entry<K, V> entry = array[i];
            array[i] = null;
            while (entry != null) {
                WeakHashMap.Entry<Object, Object> next = entry.next;
                if (entry.get() == null) {
                    entry.next = null;
                    entry.value = null;
                    --this.size;
                }
                else {
                    int index = indexFor(entry.hash, array2.length);
                    entry.next = array2[index];
                    array2[index] = entry;
                }
                entry = next;
            }
        }
    }
    
    @Override
    public void putAll(Map<? extends K, ? extends V> map) {
        int size = map.size();
        if (size == 0) {
            return;
        }
        if (size > this.threshold) {
            int n = (int)((float)size / this.loadFactor + 1.0f);
            if (n > 1073741824) {
                n = 1073741824;
            }
            int i;
            for (i = this.table.length; i < n; i <<= 1) {}
            if (i > this.table.length) {
                this.resize(i);
            }
        }
        for (Map.Entry entry : map.entrySet()) {
            this.put(entry.getKey(), entry.getValue());
        }
        return;
    }
    
    @Override
    public V remove(Object o) {
        Object maskNull = maskNull(o);
        int hash = this.hash(maskNull);
        WeakHashMap.Entry[] table = this.getTable();
        int index = indexFor(hash, table.length);
        WeakHashMap.Entry<Object, Object> entry = table[index];
        while (entry != null) {
            WeakHashMap.Entry<Object, Object> next = entry.next;
            if (hash == entry.hash && eq(maskNull, entry.get())) {
                ++this.modCount;
                --this.size;
                if (entry == entry) {
                    table[index] = next;
                }
                else {
                    entry.next = next;
                }
                return entry.value;
            }
            else {
                entry = next;
            }
        }
        return null;
    }
    
    boolean removeMapping(Object o) {
        if (!(o instanceof Map.Entry)) {
            return false;
        }
        WeakHashMap.Entry[] table = this.getTable();
        Map.Entry<Object, Object> entry = (Map.Entry)o;
        int hash = this.hash(maskNull(entry.getKey()));
        int index = indexFor(hash, table.length);
        WeakHashMap.Entry<Object, Object> entry2 = table[index];
        while (entry2 != null) {
            WeakHashMap.Entry<Object, Object> next = entry2.next;
            if (hash == entry2.hash && entry2.equals(entry)) {
                ++this.modCount;
                --this.size;
                if (entry2 == entry2) {
                    table[index] = next;
                }
                else {
                    entry2.next = next;
                }
                return true;
            }
            else {
                entry2 = next;
            }
        }
        return false;
    }
    
    @Override
    public void clear() {
        while (this.queue.poll() != null) {}
        ++this.modCount;
        Arrays.fill(this.table, null);
        this.size = 0;
        while (this.queue.poll() != null) {}
    }
    
    @Override
    public boolean containsValue(Object o) {
        if (o == null) {
            return this.containsNullValue();
        }
        WeakHashMap.Entry[] table = this.getTable();
        int length = table.length;
        while (length-- > 0) {
            WeakHashMap.Entry<Object, Object> next = table[length];
            while (next != null) {
                if (o.equals(next.value)) {
                    return true;
                }
                next = next.next;
            }
        }
        return false;
    }
    
    private boolean containsNullValue() {
        WeakHashMap.Entry[] table = this.getTable();
        int length = table.length;
        while (length-- > 0) {
            WeakHashMap.Entry<Object, Object> next = table[length];
            while (next != null) {
                if (next.value == null) {
                    return true;
                }
                next = next.next;
            }
        }
        return false;
    }
    
    @Override
    public Set<K> keySet() {
        Set<Object> keySet = this.keySet;
        if (keySet == null) {
            keySet = new WeakHashMap.KeySet();
            this.keySet = keySet;
            return keySet;
        }
        return keySet;
    }
    
    @Override
    public Collection<V> values() {
        Collection<Object> values = this.values;
        if (values == null) {
            values = new WeakHashMap.Values();
            this.values = values;
            return values;
        }
        return values;
    }
    
    @Override
    public Set<Map.Entry<K, V>> entrySet() {
        Set<Object> entrySet = this.entrySet;
        return (entrySet != null) ? entrySet : (this.entrySet = new WeakHashMap.EntrySet());
    }
    
    @Override
    public void forEach(BiConsumer<? super K, ? super V> biConsumer) {
        Objects.requireNonNull(biConsumer);
        int modCount = this.modCount;
        WeakHashMap.Entry[] table;
        for (WeakHashMap.Entry next : table = this.getTable()) {
            while (next != null) {
                Object value = next.get();
                if (value != null) {
                    biConsumer.accept(unmaskNull(value), next.value);
                }
                next = next.next;
                if (modCount != this.modCount) {
                    throw new ConcurrentModificationException();
                }
                continue;
            }
        }
    }
    
    @Override
    public void replaceAll(BiFunction<? super K, ? super V, ? extends V> biFunction) {
        Objects.requireNonNull(biFunction);
        int modCount = this.modCount;
        WeakHashMap.Entry[] table;
        for (WeakHashMap.Entry next : table = this.getTable()) {
            while (next != null) {
                Object value = next.get();
                if (value != null) {
                    next.value = biFunction.apply(unmaskNull(value), next.value);
                }
                next = next.next;
                if (modCount != this.modCount) {
                    throw new ConcurrentModificationException();
                }
                continue;
            }
        }
    }
    
    static {
        NULL_KEY = new Object();
    }
    
    private static class Entry<K, V> extends WeakReference<Object> implements Map.Entry<K, V>
    {
        V value;
        final int hash;
        WeakHashMap.Entry<K, V> next;
        
        Entry(Object referent, V value, ReferenceQueue<Object> q, int hash, WeakHashMap.Entry<K, V> next) {
            super(referent, q);
            this.value = value;
            this.hash = hash;
            this.next = next;
        }
        
        @Override
        public K getKey() {
            return WeakHashMap.unmaskNull(this.get());
        }
        
        @Override
        public V getValue() {
            return this.value;
        }
        
        @Override
        public V setValue(V value) {
            Object value2 = this.value;
            this.value = value;
            return value2;
        }
        
        @Override
        public boolean equals(Object o) {
            if (!(o instanceof Map.Entry)) {
                return false;
            }
            Map.Entry<Object, Object> entry = (Map.Entry)o;
            Object key = this.getKey();
            Object key2 = entry.getKey();
            if (key == key2 || (key != null && key.equals(key2))) {
                Object value = this.getValue();
                Object value2 = entry.getValue();
                if (value == value2 || (value != null && value.equals(value2))) {
                    return true;
                }
            }
            return false;
        }
        
        @Override
        public int hashCode() {
            return Objects.hashCode(this.getKey()) ^ Objects.hashCode(this.getValue());
        }
        
        @Override
        public String toString() {
            return this.getKey() + "=" + this.getValue();
        }
    }
    
    private abstract class HashIterator<T> implements Iterator<T>
    {
        private int index;
        private WeakHashMap.Entry<K, V> entry;
        private WeakHashMap.Entry<K, V> lastReturned;
        private int expectedModCount;
        private Object nextKey;
        private Object currentKey;
        
        HashIterator() {
            this.expectedModCount = WeakHashMap.this.modCount;
            this.index = (WeakHashMap.this.isEmpty() ? 0 : WeakHashMap.this.table.length);
        }
        
        @Override
        public boolean hasNext() {
            WeakHashMap.Entry[] table = WeakHashMap.this.table;
            while (this.nextKey == null) {
                WeakHashMap.Entry<Object, Object> entry;
                int index;
                for (entry = this.entry, index = this.index; entry == null && index > 0; entry = table[--index]) {}
                this.entry = entry;
                this.index = index;
                if (entry == null) {
                    this.currentKey = null;
                    return false;
                }
                this.nextKey = entry.get();
                if (this.nextKey == null) {
                    this.entry = this.entry.next;
                }
                else {
                    continue;
                }
            }
            return true;
        }
        
        protected WeakHashMap.Entry<K, V> nextEntry() {
            if (WeakHashMap.this.modCount != this.expectedModCount) {
                throw new ConcurrentModificationException();
            }
            if (this.nextKey == null && !this.hasNext()) {
                throw new NoSuchElementException();
            }
            this.lastReturned = this.entry;
            this.entry = this.entry.next;
            this.currentKey = this.nextKey;
            this.nextKey = null;
            return this.lastReturned;
        }
        
        @Override
        public void remove() {
            if (this.lastReturned == null) {
                throw new IllegalStateException();
            }
            if (WeakHashMap.this.modCount != this.expectedModCount) {
                throw new ConcurrentModificationException();
            }
            WeakHashMap.this.remove(this.currentKey);
            this.expectedModCount = WeakHashMap.this.modCount;
            this.lastReturned = null;
            this.currentKey = null;
            return;
        }
    }
    
    private class ValueIterator extends WeakHashMap.HashIterator<V>
    {
        @Override
        public V next() {
            return this.nextEntry().value;
        }
    }
    
    private class KeyIterator extends WeakHashMap.HashIterator<K>
    {
        @Override
        public K next() {
            return this.nextEntry().getKey();
        }
    }
    
    private class EntryIterator extends WeakHashMap.HashIterator<Map.Entry<K, V>>
    {
        @Override
        public Map.Entry<K, V> next() {
            return this.nextEntry();
        }
    }
    
    private class KeySet extends AbstractSet<K>
    {
        @Override
        public Iterator<K> iterator() {
            return new WeakHashMap.KeyIterator();
        }
        
        @Override
        public int size() {
            return WeakHashMap.this.size();
        }
        
        @Override
        public boolean contains(Object o) {
            return WeakHashMap.this.containsKey(o);
        }
        
        @Override
        public boolean remove(Object o) {
            if (WeakHashMap.this.containsKey(o)) {
                WeakHashMap.this.remove(o);
                return true;
            }
            return false;
        }
        
        @Override
        public void clear() {
            WeakHashMap.this.clear();
        }
        
        @Override
        public Spliterator<K> spliterator() {
            return new WeakHashMap.KeySpliterator(WeakHashMap.this, 0, -1, 0, 0);
        }
    }
    
    private class Values extends AbstractCollection<V>
    {
        @Override
        public Iterator<V> iterator() {
            return new WeakHashMap.ValueIterator();
        }
        
        @Override
        public int size() {
            return WeakHashMap.this.size();
        }
        
        @Override
        public boolean contains(Object o) {
            return WeakHashMap.this.containsValue(o);
        }
        
        @Override
        public void clear() {
            WeakHashMap.this.clear();
        }
        
        @Override
        public Spliterator<V> spliterator() {
            return new WeakHashMap.ValueSpliterator(WeakHashMap.this, 0, -1, 0, 0);
        }
    }
    
    private class EntrySet extends AbstractSet<Map.Entry<K, V>>
    {
        @Override
        public Iterator<Map.Entry<K, V>> iterator() {
            return new WeakHashMap.EntryIterator();
        }
        
        @Override
        public boolean contains(Object o) {
            if (!(o instanceof Map.Entry)) {
                return false;
            }
            Map.Entry<Object, Object> entry = (Map.Entry)o;
            WeakHashMap.Entry<Object, Object> entry2 = WeakHashMap.this.getEntry(entry.getKey());
            return entry2 != null && entry2.equals(entry);
        }
        
        @Override
        public boolean remove(Object o) {
            return WeakHashMap.this.removeMapping(o);
        }
        
        @Override
        public int size() {
            return WeakHashMap.this.size();
        }
        
        @Override
        public void clear() {
            WeakHashMap.this.clear();
        }
        
        private List<Map.Entry<K, V>> deepCopy() {
            ArrayList list = new ArrayList(this.size());
            Iterator<Map.Entry<K, V>> iterator = this.iterator();
            while (iterator.hasNext()) {
                list.add(new AbstractMap.SimpleEntry((Map.Entry)iterator.next()));
            }
            return list;
        }
        
        @Override
        public Object[] toArray() {
            return this.deepCopy().toArray();
        }
        
        @Override
        public <T> T[] toArray(T[] array) {
            return this.deepCopy().toArray(array);
        }
        
        @Override
        public Spliterator<Map.Entry<K, V>> spliterator() {
            return new WeakHashMap.EntrySpliterator(WeakHashMap.this, 0, -1, 0, 0);
        }
    }
    
    static class WeakHashMapSpliterator<K, V>
    {
        final WeakHashMap<K, V> map;
        WeakHashMap.Entry<K, V> current;
        int index;
        int fence;
        int est;
        int expectedModCount;
        
        WeakHashMapSpliterator(WeakHashMap<K, V> map, int index, int fence, int est, int expectedModCount) {
            this.map = map;
            this.index = index;
            this.fence = fence;
            this.est = est;
            this.expectedModCount = expectedModCount;
        }
        
        final int getFence() {
            int fence;
            if ((fence = this.fence) < 0) {
                WeakHashMap<Object, Object> map = this.map;
                this.est = map.size();
                this.expectedModCount = map.modCount;
                int length = map.table.length;
                this.fence = length;
                fence = length;
            }
            return fence;
        }
        
        public final long estimateSize() {
            this.getFence();
            return (long)this.est;
        }
    }
    
    static final class KeySpliterator<K, V> extends WeakHashMap.WeakHashMapSpliterator<K, V> implements Spliterator<K>
    {
        KeySpliterator(WeakHashMap<K, V> weakHashMap, int n, int n2, int n3, int n4) {
            super(weakHashMap, n, n2, n3, n4);
        }
        
        @Override
        public WeakHashMap.KeySpliterator<K, V> trySplit() {
            int fence = this.getFence();
            int index = this.index;
            int n = index + fence >>> 1;
            if (index >= n) {
                return null;
            }
            WeakHashMap<K, V> map;
            int n2;
            int index2;
            WeakHashMap.KeySpliterator keySpliterator = new WeakHashMap.KeySpliterator(map, n2, index2, this.est >>>= 1, this.expectedModCount);
            map = this.map;
            n2 = index;
            index2 = n;
            this.index = index2;
            return keySpliterator;
        }
        
        @Override
        public void forEachRemaining(Consumer<? super K> consumer) {
            if (consumer == null) {
                throw new NullPointerException();
            }
            WeakHashMap<Object, Object> map = this.map;
            WeakHashMap.Entry[] table = map.table;
            int fence;
            int expectedModCount;
            if ((fence = this.fence) < 0) {
                int modCount = map.modCount;
                this.expectedModCount = modCount;
                expectedModCount = modCount;
                int length = table.length;
                this.fence = length;
                fence = length;
            }
            else {
                expectedModCount = this.expectedModCount;
            }
            int index;
            if (table.length >= fence && (index = this.index) >= 0) {
                int n = index;
                int index2 = fence;
                this.index = index2;
                if (n < index2 || this.current != null) {
                    WeakHashMap.Entry<Object, Object> entry = this.current;
                    this.current = null;
                    do {
                        if (entry == null) {
                            entry = table[index++];
                        }
                        else {
                            Object value = entry.get();
                            entry = entry.next;
                            if (value != null) {
                                consumer.accept(WeakHashMap.unmaskNull(value));
                            }
                            else {
                                continue;
                            }
                        }
                    } while (entry != null || index < fence);
                }
            }
            if (map.modCount != expectedModCount) {
                throw new ConcurrentModificationException();
            }
            return;
        }
        
        @Override
        public boolean tryAdvance(Consumer<? super K> consumer) {
            if (consumer == null) {
                throw new NullPointerException();
            }
            WeakHashMap.Entry[] table = this.map.table;
            int fence;
            if (table.length >= (fence = this.getFence()) && this.index >= 0) {
                while (this.current != null || this.index < fence) {
                    if (this.current == null) {
                        this.current = table[this.index++];
                    }
                    else {
                        Object value = this.current.get();
                        this.current = this.current.next;
                        if (value != null) {
                            consumer.accept(WeakHashMap.unmaskNull(value));
                            if (this.map.modCount != this.expectedModCount) {
                                throw new ConcurrentModificationException();
                            }
                            return true;
                        }
                        else {
                            continue;
                        }
                    }
                }
            }
            return false;
        }
        
        @Override
        public int characteristics() {
            return 1;
        }
    }
    
    static final class ValueSpliterator<K, V> extends WeakHashMap.WeakHashMapSpliterator<K, V> implements Spliterator<V>
    {
        ValueSpliterator(WeakHashMap<K, V> weakHashMap, int n, int n2, int n3, int n4) {
            super(weakHashMap, n, n2, n3, n4);
        }
        
        @Override
        public WeakHashMap.ValueSpliterator<K, V> trySplit() {
            int fence = this.getFence();
            int index = this.index;
            int n = index + fence >>> 1;
            if (index >= n) {
                return null;
            }
            WeakHashMap<K, V> map;
            int n2;
            int index2;
            WeakHashMap.ValueSpliterator valueSpliterator = new WeakHashMap.ValueSpliterator(map, n2, index2, this.est >>>= 1, this.expectedModCount);
            map = this.map;
            n2 = index;
            index2 = n;
            this.index = index2;
            return valueSpliterator;
        }
        
        @Override
        public void forEachRemaining(Consumer<? super V> consumer) {
            if (consumer == null) {
                throw new NullPointerException();
            }
            WeakHashMap<Object, Object> map = this.map;
            WeakHashMap.Entry[] table = map.table;
            int fence;
            int expectedModCount;
            if ((fence = this.fence) < 0) {
                int modCount = map.modCount;
                this.expectedModCount = modCount;
                expectedModCount = modCount;
                int length = table.length;
                this.fence = length;
                fence = length;
            }
            else {
                expectedModCount = this.expectedModCount;
            }
            int index;
            if (table.length >= fence && (index = this.index) >= 0) {
                int n = index;
                int index2 = fence;
                this.index = index2;
                if (n < index2 || this.current != null) {
                    WeakHashMap.Entry<Object, Object> entry = this.current;
                    this.current = null;
                    do {
                        if (entry == null) {
                            entry = table[index++];
                        }
                        else {
                            Object value = entry.get();
                            Object value2 = entry.value;
                            entry = entry.next;
                            if (value != null) {
                                consumer.accept(value2);
                            }
                            else {
                                continue;
                            }
                        }
                    } while (entry != null || index < fence);
                }
            }
            if (map.modCount != expectedModCount) {
                throw new ConcurrentModificationException();
            }
            return;
        }
        
        @Override
        public boolean tryAdvance(Consumer<? super V> consumer) {
            if (consumer == null) {
                throw new NullPointerException();
            }
            WeakHashMap.Entry[] table = this.map.table;
            int fence;
            if (table.length >= (fence = this.getFence()) && this.index >= 0) {
                while (this.current != null || this.index < fence) {
                    if (this.current == null) {
                        this.current = table[this.index++];
                    }
                    else {
                        Object value = this.current.get();
                        Object value2 = this.current.value;
                        this.current = this.current.next;
                        if (value != null) {
                            consumer.accept(value2);
                            if (this.map.modCount != this.expectedModCount) {
                                throw new ConcurrentModificationException();
                            }
                            return true;
                        }
                        else {
                            continue;
                        }
                    }
                }
            }
            return false;
        }
        
        @Override
        public int characteristics() {
            return 0;
        }
    }
    
    static final class EntrySpliterator<K, V> extends WeakHashMap.WeakHashMapSpliterator<K, V> implements Spliterator<Map.Entry<K, V>>
    {
        EntrySpliterator(WeakHashMap<K, V> weakHashMap, int n, int n2, int n3, int n4) {
            super(weakHashMap, n, n2, n3, n4);
        }
        
        @Override
        public WeakHashMap.EntrySpliterator<K, V> trySplit() {
            int fence = this.getFence();
            int index = this.index;
            int n = index + fence >>> 1;
            if (index >= n) {
                return null;
            }
            WeakHashMap<K, V> map;
            int n2;
            int index2;
            WeakHashMap.EntrySpliterator entrySpliterator = new WeakHashMap.EntrySpliterator(map, n2, index2, this.est >>>= 1, this.expectedModCount);
            map = this.map;
            n2 = index;
            index2 = n;
            this.index = index2;
            return entrySpliterator;
        }
        
        @Override
        public void forEachRemaining(Consumer<? super Map.Entry<K, V>> consumer) {
            if (consumer == null) {
                throw new NullPointerException();
            }
            WeakHashMap<Object, Object> map = this.map;
            WeakHashMap.Entry[] table = map.table;
            int fence;
            int expectedModCount;
            if ((fence = this.fence) < 0) {
                int modCount = map.modCount;
                this.expectedModCount = modCount;
                expectedModCount = modCount;
                int length = table.length;
                this.fence = length;
                fence = length;
            }
            else {
                expectedModCount = this.expectedModCount;
            }
            int index;
            if (table.length >= fence && (index = this.index) >= 0) {
                int n = index;
                int index2 = fence;
                this.index = index2;
                if (n < index2 || this.current != null) {
                    WeakHashMap.Entry<Object, Object> entry = this.current;
                    this.current = null;
                    do {
                        if (entry == null) {
                            entry = table[index++];
                        }
                        else {
                            Object value = entry.get();
                            Object value2 = entry.value;
                            entry = entry.next;
                            if (value != null) {
                                consumer.accept(new AbstractMap.SimpleImmutableEntry(WeakHashMap.unmaskNull(value), value2));
                            }
                            else {
                                continue;
                            }
                        }
                    } while (entry != null || index < fence);
                }
            }
            if (map.modCount != expectedModCount) {
                throw new ConcurrentModificationException();
            }
            return;
        }
        
        @Override
        public boolean tryAdvance(Consumer<? super Map.Entry<K, V>> consumer) {
            if (consumer == null) {
                throw new NullPointerException();
            }
            WeakHashMap.Entry[] table = this.map.table;
            int fence;
            if (table.length >= (fence = this.getFence()) && this.index >= 0) {
                while (this.current != null || this.index < fence) {
                    if (this.current == null) {
                        this.current = table[this.index++];
                    }
                    else {
                        Object value = this.current.get();
                        Object value2 = this.current.value;
                        this.current = this.current.next;
                        if (value != null) {
                            consumer.accept(new AbstractMap.SimpleImmutableEntry(WeakHashMap.unmaskNull(value), value2));
                            if (this.map.modCount != this.expectedModCount) {
                                throw new ConcurrentModificationException();
                            }
                            return true;
                        }
                        else {
                            continue;
                        }
                    }
                }
            }
            return false;
        }
        
        @Override
        public int characteristics() {
            return 1;
        }
    }
}
